##############################################################################
##############################################################################
#                               ChooseLinks.tcl
##############################################################################
##############################################################################
# In this namespace are the procedures that implement the 'Choose Links'
# dialog.
##############################################################################
##############################################################################
# Copyright 2001-2007 Andrs Garca Garca  -- fandom@telefonica.net
# Distributed under the terms of the GPL v2
##############################################################################
##############################################################################

namespace eval ChooseLinks {


image create bitmap mas -data "
    #define mas_width 16
    #define mas_height 16
    static unsigned char mas_bits[] = {
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x80, 0x01,
        0x80, 0x01, 0xf0, 0x0f, 0xf0, 0x0f, 0x80, 0x01, 0x80, 0x01, 0x80, 0x01,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
"
image create bitmap menos -data "
    #define menos_width 16
    #define menos_height 16
    static unsigned char menos_bits[] = {
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0xf0, 0x0f, 0xf0, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
"

###############################################################################
# Traces
#   When one of the options changes this procedure is invoked to change the
#   links.
###############################################################################
proc Traces {linkArray name1 name2 op} {
    variable traces
    global downOptions

    if {$downOptions($name2)!=$traces($name2)} {
        ChooseLinks::ChangeLinksToChoose $linkArray
        set traces($name2) $downOptions($name2)
    }

    return
}

###############################################################################
# SetChosenLinks
#    Puts into getleftState(chosenLinks) the links the user wants to download.
###############################################################################
proc SetChosenLinks {} {
    global downOptions getleftState

    if {($downOptions(map)==1)&&($getleftState(mapDone)==1)} {
        set getleftState(mapLinks) $getleftState(chosenLinks)
    }
    set getleftState(chosenLinks) ""
    for {set i 1} {$i<$HtmlParser::nLinks} {incr i} {
        lappend getleftState(chosenLinks) $HtmlParser::links($i,ok)
        if {$HtmlParser::links($i,ok)==0} {
            set ::HtmlParser::linksToIgnore([string tolower $HtmlParser::links($i,url)]) 1
        }
    }
    return
}


###############################################################################
# CopyLink
#   Copies the link to the clipboard
###############################################################################
proc CopyLink {} {
    variable activeLink

    regexp {[0-9]+$} $activeLink link
    clipboard clear
    for {set i 1;set j 0} {$i<$HtmlParser::nLinks} {incr i} {
        incr j $HtmlParser::links($i,ok)
        if {$j==$link} {
            clipboard append $HtmlParser::links($i,url)
            break
        }
    }
    return
}

###############################################################################
# PutInQueue
#   Puts one of the links into the queue.
#
# Parameter
#    textPath: Path of the text widget where the links are.
#    linkArray: The array with the links.
###############################################################################
proc PutInQueue {textPath linkArray} {
    variable win
    variable activeLink

    upvar #0 $linkArray links

    regexp {[0-9]+} $activeLink line
    for {set i 1;set j 0} {[info exists links($i,url)]} {incr i} {
        incr j $links($i,shown)
        if {$j==$line} {
            set urlToDownload $links($i,url)
            break
        }
    }

    EnterUrl::EnterUrl 0 $win(top) $urlToDownload

    return
}

###############################################################################
# PutAllInQueue
#   Puts all the selected links into the queue.
#
# Parameter
#    textPath: Path of the text widget where the links are.
#    linkArray: The array with the links.
###############################################################################
proc PutAllInQueue {textPath linkArray} {
    global   dirGetleft
    variable win

    upvar #0 $linkArray links

    set baseDir [Dialogos::SelectDirectory                                  \
            [file dirname $dirGetleft(lastDir)] $win(top)]

    for {set i 1} {[info exists links($i,url)]} {incr i} {
        if {$links($i,ok)==1} {
            set descrip $links($i,descrip)
            regsub {(.*: )} $descrip {} descrip
            set dir [file join $baseDir $descrip]
            PutUrlInQueue $links($i,url) $dir [array get ::downOptions]
        }
    }
    return
}

###############################################################################
# ChooseAllOptions
#    Invokes the dialog to choose from all the downloading options, if
#    necessary it also makes sure that the links are changed.
#
# Parameter
#    The name of the array with the links
###############################################################################
proc ChooseAllOptions {linkArray} {
    global   downOptions
    variable win

    if {[Opciones::ChooseOptions downOptions $win(top)]==1} {
        ChooseLinks::ChangeLinksToChoose $linkArray
    }

    return
}

###############################################################################
# MenuKey
#    When the user preses the context menu key, this will pop the context menu
#    up.
###############################################################################
proc MenuKey {X Y} {
    variable win

    $win(menu) entryconfigure 5 -state disabled
    $win(menu) entryconfigure 6 -state disabled
    $win(menu) entryconfigure 8 -state disabled
    $win(menu) entryconfigure 9 -state disabled

    tk_popup $win(menu) $X $Y

    return
}

###############################################################################
# ContextMenu
#    When the user right-clicks in the dialog, this will pop the context menu
#    up.
#
# Parameters:
#    link: The nearest link to the click.
#    x,y: coords where to show the menu
###############################################################################
proc ContextMenu {link x y} {
    variable win
    variable activeLink
    global   getleftOptions getleftState

    if {[winfo exists $link]} {
        $win(menu) entryconfigure 8 -state normal
        $win(menu) entryconfigure 9 -state normal
    } else {
        $win(menu) entryconfigure 8 -state disabled
        $win(menu) entryconfigure 9 -state disabled
    }
    set activeLink $link

    $win(menu) entryconfigure 5 -state normal
    $win(menu) entryconfigure 6 -state normal

    tk_popup $win(menu) $x $y

    return
}

###############################################################################
# Chosen
#    Selects and deselects the chosen files.
#
# Parameters:
#    pattern: regular expresion that identifies the files to (de)select.
#    select: '1' to select, '0' to deselect.
###############################################################################
proc Chosen {linkArray pattern select} {
    global downOptions

    set pattern [subst $pattern]
    if {$pattern=="(\$)"} return
    upvar #0 $linkArray links

    for {set i 1} {[info exists links($i,url)]} {incr i} {
        if {$links($i,shown)==0} continue
        if {[regexp -nocase $pattern $links($i,file)]} {
            set links($i,ok) $select
        }
    }
    return
}

###############################################################################
# ChooseGroup
#    Activates or deactivates the links that are either before or after the
#    link where the user right-clicked.
#
# Parameters:
#    which: 'pre' the links before the click.
#           'fol' the links after.
#    action: '0' deactivates the links
#            '1' activates them.
###############################################################################
proc ChooseGroup {which action} {
    variable win
    variable activeLink

    regexp {[0-9]+} $activeLink buttonIndex
    if {$which=="pre"} {
        set next -1
    } else {
        set next  1
    }
    for {set i $buttonIndex} {1} {incr i $next} {
        if {![winfo exists $win(links).enlaces$i]} {
            if {($which=="pre")} {
                if {$i==0} return else continue
                } else return
        }
        if {$action==0} {
            $win(links).enlaces$i state !selected
        } else {
            $win(links).enlaces$i state  selected
        }
    }
    return
}

###############################################################################
# ChangeLinksToChoose
#    If the user changes the downloading options in the 'Choose Files' dialog
#    this procedure takes care of displaying the new links to choose from.
#
# Parameter:
#    linkArray: The name of the array where the links are stored.
###############################################################################
proc ChangeLinksToChoose {linkArray} {
    global directories
    variable win
    variable refererUrl

    $win(canvas) configure -cursor watch
    $win(canvas) configure -state  normal
    destroy $win(links)
    update

    catch {unset ::Descargar::filesProcessed}
    set refererFile [::Commands::UrlToFile $refererUrl $directories(base)]

    HtmlParser::FilterLinks $refererUrl $linkArray 0
    Commands::ChangePage $refererUrl
    LinksToChoose $linkArray

    $win(canvas) configure -state  disabled
    $win(canvas) configure -cursor left_ptr
    tkwait visibility $win(links)
    $win(canvas) config -scrollregion "0 0                                    \
            [winfo reqwidth $win(links)] [winfo reqheight $win(links)]"
    ChangeFrameSize
    return
}

###############################################################################
# LinksToChoose
#    Inserts the links in the 'Choose files' directory.
#
# Parameter
#    linkArray: Array where the links to be shown are stored.
###############################################################################
proc LinksToChoose {linkArray} {
    global downOptions getleftState labelMessages
    global getleftOptions labelTitles
    variable win

    upvar #0 $linkArray links

    if {($downOptions(map)==1)&&($getleftState(mapDone)==1)} {
        set map 1
    } else {
        set map 0
    }
    set linkFrame [ttk::frame $win(canvas).links]
    set win(linksId) [$win(canvas) create window 0 0 -anchor nw -window $linkFrame]
    set win(links)   $linkFrame

    for {set i 1 ; set j 1 ; set pre ""} {[info exists links($i,url)]} {incr i} {
        set links($i,shown) 0
        if {$links($i,ok)==0} continue
        set links($i,shown) 1
        set links($i,descrip) [HtmlParser::TidyDescription                      \
                $links($i,descrip) $links($i,url)]
        if {$map==1} {
            for {set k 0 ; set spc "   " ; set pre ""} {$k<$links($i,level)}    \
                    {incr k} {
                append pre $spc
            }
            set level $links($i,level)
        } else {
            set level 0
        }
        set linkButton [ttk::checkbutton $linkFrame.enlaces$j                   \
                -variable ${linkArray}($i,ok) -text $links($i,descrip)]

        pack $linkButton -side top -fill x -padx [expr $level*10]

        bind $linkButton <<ContextButton>> {
            ChooseLinks::ContextMenu %W %X %Y
        }
        incr j
        update idletasks
    }

    if {$j==1} {
        ttk::label $linkFrame.noLinks -textvariable labelMessages(noLinksOpt)   \
                -anchor w -justify left
        pack $linkFrame.noLinks -side top -fill x
    }

    bind $linkFrame <<ContextButton>>       "
        ChooseLinks::ContextMenu $linkFrame.enlaces$j %X %Y
    "

    grid columnconfigure $win(canvas) 0 -weight 1
    grid rowconfigure    $win(canvas) 0 -weight 1

    return
}

###############################################################################
# SetEncodingMenus
#    Creates the menus to choose from the different encodings
#
# Parameter
#    menu: The menu to create.
#    encodings: A list with the encodings to add.
#    referer: The page  from which we got the links.
#    linkArray: Name of the array with the links.
###############################################################################
proc SetEncodingMenus {menu encodings referer linkArray} {
    global charSets

    foreach codeName $encodings {
        if {![regexp {(?:\()(.*)(?:\))} $codeName nada code]} {
            set code $codeName
        }
        if {![info exists charSets($code)]} {
            continue
        }
        set code [string tolower $code]
        $menu add radio -variable ChooseLinks::tmp -value $code    \
                -label $codeName -command "
                    Descargar::Preprocessing $referer 0
                    HtmlParser::ChangeEncoding \$charSets(\$ChooseLinks::tmp)
                    ChooseLinks::ChangeLinksToChoose $linkArray
                    set ChooseLinks::tmp {}
                "
    }
    return
}

###############################################################################
# ChangeFrameSize
#    When the toplevel window is resized, this procedure updates the size of
#    the frame where the checkbuttons are contained.
###############################################################################
proc ChangeFrameSize {} {
    variable win

    set reqHeight [winfo reqheight $win(links)]
    set reqWidth  [winfo reqwidth  $win(links)]

    set canvasHeight [$win(canvas) cget -height]
    set canvasWidth  [$win(canvas) cget -width]

    if {$reqHeight>[expr $canvasHeight * 2]} {
        set height $reqHeight
    } else {
        set height [expr $canvasHeight * 5]
    }
    if {$reqWidth>[expr $canvasWidth * 2]} {
        set width $reqWidth
    } else {
        set width  [expr $canvasWidth  * 5]
    }

    $win(canvas) itemconfigure $win(linksId) -height $height
    $win(canvas) itemconfigure $win(linksId) -width  $width

    return
}

###############################################################################
# ScrollLinkFrame
#    Prevents the linkframe from moving in the canvas when it shouldn't.
#
# Parameter
#    height, width: New size of the canvas.
###############################################################################
proc ScrollLinkFrame {up how} {
    variable win
    if {$up<0} {
        if {[lindex [$ChooseLinks::win(canvas) yview] 0]==0} {
            return
        }
    } else {
        if {[lindex [$ChooseLinks::win(canvas) yview] 1]==1} {
            return
        }
    }
    $ChooseLinks::win(canvas) yview scroll $up $how 
    return
}

###############################################################################
# ChooseDownloads
#    Shows a dialog with all the links in the entered Url so you can choose
#    which ones will be followed and which will be ignored.
#
# Parameter
#    referer: url of the page from which the links where taken.
#    linkArray: Array where the links to be shown are stored.
###############################################################################
proc ChooseDownloads {referer linkArray} {
    global labelButtons labelTitles labelDialogs labelMenus indexButtons
    global downOptions getleftState getleftOptions indexMenus dirGetleft
    variable refererUrl $referer
    variable traces
    variable win
    variable done
    variable menu
    variable taskbar

    catch {wm withdraw $Ventana::window(top)}

    if {$HtmlParser::nLinks==1} "return 0"

    set win(top) [toplevel .elegir]
    keynav::enableMnemonics $win(top)

    set extFrame      [ttk::frame $win(top).extFrame -class SquareExternalFrame]
    set marco         [ttk::frame $extFrame.marco -relief sunken -borderwidth 2]
    set taskbarFrame  [ttk::frame $extFrame.taskbarFrame]
    set taskbarLeft   [ttk::frame $taskbarFrame.left]
    set taskbarCenter [ttk::frame $taskbarFrame.center]
    set taskbarRight  [ttk::frame $taskbarFrame.right]
    set checks        [ttk::frame $extFrame.checks]

    wm title $win(top) "$labelTitles(choose): $referer"

    set canvas [canvas $marco.canvas -bg $getleftOptions(menus)                 \
            -highlightthickness 0                                               \
            -yscrollcommand [list $marco.yscroll set]                           \
            -xscrollcommand [list $marco.xscroll set]]
    set win(canvas) $canvas

    set barray [ttk::scrollbar $marco.yscroll -orient vertical                  \
            -command [list $canvas yview]]
    set barrax [ttk::scrollbar $marco.xscroll -orient horizontal                \
            -command [list $canvas xview]]

    set taskbar(seeUrl) [ttk::button $taskbarLeft.seeUrl                        \
            -image $iconNS::iconImages(seeUrl)                                  \
            -style Toolbutton -command  "
                Ayuda::InvokeBrowser $referer $win(top)
            "]
    ::BalloonHelp::set_balloon $taskbar(seeUrl)  $labelDialogs(seeUrl)

    set taskbar(conf)   [ttk::button $taskbarLeft.conf                          \
            -image $iconNS::iconImages(conf)                                    \
            -style Toolbutton -command  "
                ChooseLinks::ChooseAllOptions $linkArray
            "]
    ::BalloonHelp::set_balloon $taskbar(conf)   $labelDialogs(options)

    set taskbar(downld) [ttk::button $taskbarCenter.download                    \
            -image $iconNS::iconImages(downld)                                  \
            -style Toolbutton -command  "
                mainWin::UpdateUrlOptions
                ChooseLinks::SetChosenLinks
                set ChooseLinks::done 1
            "]
    ::BalloonHelp::set_balloon $taskbar(downld)   $labelDialogs(download)

    set taskbar(help)   [ttk::button $taskbarRight.help                         \
            -image $iconNS::iconImages(help)                                    \
            -style Toolbutton -command  "Ayuda::Ayuda howtoDown#ChooseLinks"]
    ::BalloonHelp::set_balloon $taskbar(help) $labelDialogs(help)
    set taskbar(exit)   [ttk::button $taskbarRight.exit                         \
            -image $iconNS::iconImages(exit)                                    \
            -style Toolbutton -command  "
                set ChooseLinks::done 0
            "]
    ::BalloonHelp::set_balloon $taskbar(exit) $labelDialogs(exit)

    set lhtml [ttk::label  $checks.lhtml -text "  $labelMenus(html)"]
    set htmla [ttk::button $checks.htmla -image mas     -style Toolbutton      \
            -command "ChooseLinks::Chosen $linkArray {(html$)|(htm$)|(/$)} 1"]
    set htmle [ttk::button $checks.htmle -image menos   -style Toolbutton      \
            -command "ChooseLinks::Chosen $linkArray {(html$)|(htm$)|(/$)} 0"]
    set lima  [ttk::label $checks.lima   -text "  $labelMenus(images)"]
    set imaa  [ttk::button $checks.imaa  -image mas     -style Toolbutton      \
            -command "ChooseLinks::Chosen $linkArray {(gif$)|(jpg$)|(jpeg$)|(bmp$)|(xbm$)|(tiff?$)|(png$)} 1"]
    set imae  [ttk::button $checks.imae  -image menos   -style Toolbutton      \
            -command "ChooseLinks::Chosen $linkArray {(gif$)|(jpg$)|(jpeg$)|(bmp$)|(xbm$)|(tiff?$)|(png$)} 0"]
    set lcom  [ttk::label $checks.lcom   -text "  $labelMenus(archives)"]
    set coma  [ttk::button $checks.cona  -image mas     -style Toolbutton      \
            -command "ChooseLinks::Chosen $linkArray {(tar$)|(gz$)|(z$)|(zip$)|(tgz$)|(rpm$)|(deb$)|(sit$)} 1"]
    set come  [ttk::button $checks.cone  -image menos   -style Toolbutton      \
            -command "ChooseLinks::Chosen $linkArray {(tar$)|(gz$)|(z$)|(zip$)|(tgz$)|(rpm$)|(deb$)|(sit$)} 0"]
    set ltodo [ttk::label $checks.ltodo  -text "  $labelMenus(allFiles)"]
    set todoa [ttk::button $checks.todoa -image mas    -style Toolbutton       \
            -command "ChooseLinks::Chosen $linkArray {.} 1"]
    set todoe [ttk::button $checks.todoe -image menos  -style Toolbutton       \
            -command "ChooseLinks::Chosen $linkArray {.} 0"]
    set lany  [ttk::label $checks.lany   -text "  "]
    set eany  [ttk::entry $checks.eany   -width 4                              \
            -textvariable ChooseLinks::anyExtension]
    set ChooseLinks::anyExtension ""
    set anya [ttk::button $checks.anya -image mas    -style Toolbutton         \
            -command "ChooseLinks::Chosen $linkArray {(\$ChooseLinks::anyExtension\$)} 1"]
    set anye [ttk::button $checks.anye -image menos  -style Toolbutton         \
            -command "ChooseLinks::Chosen $linkArray {(\$ChooseLinks::anyExtension\$)} 0"]

    wm geometry $win(top) 600x600+150+60
    wm minsize $win(top)  470 250
    tkwait visibility $win(top)

    LinksToChoose $linkArray

    $win(canvas) config -scrollregion 

    set menu [menu $win(top).menu -tearoff 0 -bg $getleftOptions(menus)        \
            -font $getleftOptions(menuFont)]
    regsub {bold} $getleftOptions(menuFont) {} getleftOptions(menuFont)

    set win(menu) $menu
    $menu add cascade -label "$labelMenus(html)"      -menu $menu.html         \
            -underline $indexMenus(html)
    $menu add cascade -label "$labelMenus(images)"    -menu $menu.images       \
            -underline $indexMenus(images)
    $menu add cascade -label "$labelMenus(archives)"  -menu $menu.archives     \
            -underline $indexMenus(archives)
    $menu add cascade -label "$labelMenus(all)" -menu $menu.all                \
            -underline $indexMenus(all)
    $menu add separator
    $menu add cascade -label "$labelMenus(previous)"  -menu $menu.previous     \
            -underline $indexMenus(previous)
    $menu add cascade -label "$labelMenus(following)" -menu $menu.following    \
            -underline $indexMenus(following)
    $menu add separator
    $menu add command -label "$labelMenus(copyLink)"                           \
            -underline $indexMenus(copyLink)                                   \
            -command "ChooseLinks::CopyLink"
    $menu add command -label "$labelMenus(queue)"                              \
            -underline $indexMenus(queue)                                      \
            -command "ChooseLinks::PutInQueue    $canvas $linkArray"
    $menu add command -label "$labelMenus(allQueue)"                           \
            -underline $indexMenus(allQueue)                                   \
            -command "ChooseLinks::PutAllInQueue $canvas $linkArray"
    $menu add separator
    $menu add cascade -menu $menu.encodings                                    \
            -label $labelMenus(encodings)  -underline $indexMenus(encodings)
    $menu add cascade -label "$labelMenus(options)" -menu  $menu.options       \
            -underline $indexMenus(options)

    menu $menu.html      -tearoff 0 -font $getleftOptions(menuFont) -bg $getleftOptions(menus)
    menu $menu.images    -tearoff 0 -font $getleftOptions(menuFont) -bg $getleftOptions(menus)
    menu $menu.archives  -tearoff 0 -font $getleftOptions(menuFont) -bg $getleftOptions(menus)
    menu $menu.all       -tearoff 0 -font $getleftOptions(menuFont) -bg $getleftOptions(menus)
    menu $menu.previous  -tearoff 0 -font $getleftOptions(menuFont) -bg $getleftOptions(menus)
    menu $menu.following -tearoff 0 -font $getleftOptions(menuFont) -bg $getleftOptions(menus)
    menu $menu.options   -tearoff 0 -font $getleftOptions(menuFont) -bg $getleftOptions(menus)

    $menu.html      add command -label "$labelMenus(activate)"                 \
            -command "$htmla invoke" -underline $indexMenus(activate)
    $menu.html      add command -label "$labelMenus(deactivate)"               \
            -command "$htmle invoke" -underline $indexMenus(deactivate)
    $menu.images    add command -label "$labelMenus(activate)"                 \
            -command "$imaa  invoke" -underline $indexMenus(activate)
    $menu.images    add command -label "$labelMenus(deactivate)"               \
            -command "$imae  invoke" -underline $indexMenus(deactivate)
    $menu.archives  add command -label "$labelMenus(activate)"                 \
            -command "$coma  invoke" -underline $indexMenus(activate)
    $menu.archives  add command -label "$labelMenus(deactivate)"               \
            -command "$come  invoke" -underline $indexMenus(deactivate)
    $menu.all       add command -label "$labelMenus(activate)"                 \
            -command "$todoa invoke" -underline $indexMenus(activate)
    $menu.all       add command -label "$labelMenus(deactivate)"               \
            -command "$todoe invoke" -underline $indexMenus(deactivate)
    $menu.previous  add command -label "$labelMenus(activate)"                 \
            -command "ChooseLinks::ChooseGroup pre 1"                          \
            -underline $indexMenus(activate)
    $menu.previous  add command -label "$labelMenus(deactivate)"               \
            -command "ChooseLinks::ChooseGroup pre 0"                          \
            -underline $indexMenus(deactivate)
    $menu.following add command -label "$labelMenus(activate)"                 \
            -command "ChooseLinks::ChooseGroup fol 1"                          \
            -underline $indexMenus(activate)
    $menu.following add command -label "$labelMenus(deactivate)"               \
            -command "ChooseLinks::ChooseGroup fol 0"                          \
            -underline $indexMenus(deactivate)

    $menu.options add command                                                  \
            -label $labelMenus(all)        -underline $indexMenus(all)         \
            -command "ChooseLinks::ChooseAllOptions $linkArray"
    $menu.options add cascade -menu $menu.options.directorios                  \
            -label $labelMenus(uplinks)    -underline $indexMenus(uplinks)
    $menu.options add cascade -menu $menu.options.levels                       \
            -label $labelMenus(levels)     -underline $indexMenus(levels)
    $menu.options add cascade -menu $menu.options.exLevels                     \
            -label $labelMenus(external)   -underline $indexMenus(external)
    $menu.options add cascade -menu $menu.options.filters                      \
            -label $labelMenus(fileFilter) -underline $indexMenus(fileFilter)
    $menu.options add check   -variable downOptions(update)                    \
            -label $labelMenus(update)     -underline $indexMenus(update)      \
            -onvalue 1 -offvalue 0
    $menu.options add check   -variable downOptions(cgi)                       \
            -label $labelMenus(cgi)        -underline $indexMenus(cgi)         \
            -onvalue 1 -offvalue 0

    set menuDir [menu $menu.options.directorios -tearoff 0                     \
            -bg $getleftOptions(menus) -font $getleftOptions(menuFont)]
    $menuDir add radio -variable downOptions(dir)  -value 1                    \
           -label $labelMenus(linksFollow)  -underline $indexMenus(linksFollow)
    $menuDir add radio -variable downOptions(dir)  -value 0                    \
            -label $labelMenus(linksIgnore) -underline $indexMenus(linksIgnore)

    set menuLev [menu $menu.options.levels -tearoff 0                          \
            -bg $getleftOptions(menus) -font $getleftOptions(menuFont)]
    $menuLev add radio -variable downOptions(levels) -value -1                 \
            -label $labelMenus(noLimit) -underline $indexMenus(noLimit)
    for {set i 0} {$i<6} {incr i} {
        $menuLev add radio -label $i -variable downOptions(levels)             \
                -value $i
    }

    set menuEx [menu $menu.options.exLevels -tearoff 0                         \
            -bg $getleftOptions(menus) -font $getleftOptions(menuFont)]
    $menuEx add radio -variable downOptions(exLevels) -value 0                 \
            -label $labelMenus(linksIgnore) -underline $indexMenus(linksIgnore)\
            -command "
                set downOptions(external) 0
            "
    for {set i 1} {$i<4} {incr i} {
        $menuEx add radio -label $i -variable downOptions(exLevels)            \
            -value $i -command "
                set downOptions(external) 1
            "
    }

    set menuFil [menu $menu.options.filters -tearoff 0                         \
            -bg $getleftOptions(menus) -font $getleftOptions(menuFont)]
    $menuFil add check -variable downOptions(onlyHtml) -onvalue 1 -offvalue 0  \
            -label $labelMenus(onlyHtml) -underline $indexMenus(onlyHtml)
    $menuFil add cascade -menu $menuFil.images                                 \
            -label $labelMenus(images)   -underline $indexMenus(images)
    $menuFil add command -label $labelMenus(chooseFilter) -command "
        Herramientas::FilterFiles \"$downOptions(filter)\" $win(top)

        ChooseLinks::ChangeLinksToChoose $linkArray
    "

    set menuFilImage [menu $menuFil.images -tearoff 0                          \
            -bg $getleftOptions(menus) -font $getleftOptions(menuFont)]
    $menuFilImage add radio -variable downOptions(images) -value 0             \
            -label $labelMenus(allImages)   -underline $indexMenus(allImages)
    $menuFilImage add radio -variable downOptions(images) -value 1             \
            -label $labelMenus(onlyThumb) -underline $indexMenus(onlyThumb)
    $menuFilImage add radio -variable downOptions(images) -value 2             \
            -label $labelMenus(noThumb)   -underline $indexMenus(noThumb)

    set menuEncodings [menu $menu.encodings -tearoff 0                         \
            -bg $getleftOptions(menus) -font $getleftOptions(menuFont)]
    $menuEncodings add cascade -label $labelMenus(europa)                      \
            -underline $indexMenus(europa) -menu $menuEncodings.europa
    $menuEncodings add cascade -label $labelMenus(asia)                        \
            -underline $indexMenus(asia)   -menu $menuEncodings.asia
    $menuEncodings add cascade -label $labelMenus(africa)                      \
            -underline $indexMenus(africa) -menu $menuEncodings.africa
    set encodings [lsort "
        \"$labelMenus(ascii)\"                     
        \"${labelMenus(westEuropa)} (iso8859-1)\"   \"${labelMenus(westEuropa)} (cp850)\"
        \"${labelMenus(westEuropa)} (macRoman)\"    \"${labelMenus(celtic)} (iso8859-7)\"
        \"${labelMenus(greek)} (iso8859-7)\"        \"${labelMenus(greek)} (macGreek)\"
        \"${labelMenus(iceland)} (macIceland)\"     \"${labelMenus(baltic)} (iso8859-4)\"
        \"${labelMenus(baltic)}\"                   \"${labelMenus(baltic)} (cp1257)\"
        \"${labelMenus(centerEuropa)} (cp852)\"     \"${labelMenus(centerEuropa)} (cp1250)\"
        \"${labelMenus(centerEuropa)} (iso8859-3)\" \"${labelMenus(centerEuropa)} (iso8859-2)\"
        \"${labelMenus(croatian)} (macCroatian)\"   \"${labelMenus(cyrillic)} (cp855)\"
        \"${labelMenus(cyrillic)} (macCyrillic)\"   \"${labelMenus(cyrillic)} (macUkraine)\"
        \"${labelMenus(cyrillic)} (iso8859-5)\"     \"${labelMenus(cyrillic)} (cp866)\"
        \"${labelMenus(cyrillic)} (koi8-r)\"
    "]
    set $menuEncodings.europa [menu $menuEncodings.europa -tearoff 0           \
            -bg $getleftOptions(menus) -font $getleftOptions(menuFont)]
    SetEncodingMenus $menuEncodings.europa $encodings $referer $linkArray
    set encodings [lsort "
        \"${labelMenus(ascii)}\"               \"${labelMenus(simpleChin)} (gb2312)\"
        \"${labelMenus(tradChin)} (big5)\"     \"${labelMenus(japan)} (iso2022-jp)\"
        \"${labelMenus(japan)} (euc-jp)\"      \"${labelMenus(japan)} (shiftjis)\"
        \"${labelMenus(korean)} (euc-kr)\"     \"${labelMenus(turkish)} (cp857)\"
        \"${labelMenus(turkish)} (iso8859-9)\" \"${labelMenus(turkish)} (macTurkish)\"
        \"${labelMenus(hebrew)} (cp862)\"      \"${labelMenus(hebrew)} (iso8859-8)\"
        \"${labelMenus(arabic)} (cp864)\"      \"${labelMenus(arabic)} (cp1256)\"
        \"${labelMenus(unicode)} (utf-8)\"
    "]
    set $menuEncodings.asia [menu $menuEncodings.asia -tearoff 0              \
            -bg $getleftOptions(menus) -font $getleftOptions(menuFont)]
    SetEncodingMenus $menuEncodings.asia $encodings $referer $linkArray
    set encodings [lsort "
        \"${labelMenus(ascii)}\"            \"${labelMenus(arabic)} (cp864)\"
        \"${labelMenus(arabic)} (cp1256)\"  \"${labelMenus(unicode)} (utf-8)\"
    "]
    set $menuEncodings.africa [menu $menuEncodings.africa -tearoff 0          \
            -bg $getleftOptions(menus) -font $getleftOptions(menuFont)]
    SetEncodingMenus $menuEncodings.africa $encodings $referer $linkArray

    bind $win(top) <KeyPress-Prior> "ChooseLinks::ScrollLinkFrame -1  pages"
    bind $win(top) <KeyPress-Next>  "ChooseLinks::ScrollLinkFrame  1 pages" 
    bind $win(top) <Key-Up>         "ChooseLinks::ScrollLinkFrame -1 units"
    bind $win(top) <Key-Down>       "ChooseLinks::ScrollLinkFrame  1 units"
    bind $win(top) <MouseWheel> "
        if {%D<0} {
            ChooseLinks::ScrollLinkFrame  3 units
        } else {
            ChooseLinks::ScrollLinkFrame -3 units
        }
    "
    bind $win(top) <Escape>    "$taskbar(exit) invoke"
    bind $win(top) <Button-4>  "ChooseLinks::ScrollLinkFrame -3 units"
    bind $win(top) <Button-5>  "ChooseLinks::ScrollLinkFrame  3 units"
    bind $win(top) <<MenuKey>> "ChooseLinks::MenuKey %X %Y"

    bind $win(canvas) <Configure> "ChooseLinks::ChangeFrameSize"

    if {$::tcl_version>=8.5} {
        grid anchor $extFrame c
        grid anchor $marco    c
        grid anchor $checks   c
        grid anchor $taskbarFrame  c
    }

    grid $extFrame       -sticky news
    grid $taskbarFrame   -sticky ew   -padx 12 -pady 3
    grid $taskbarLeft    -sticky w
    grid $taskbarCenter  -row 0 -column 1   -sticky ew
    grid $taskbarRight   -row 0 -column 2   -sticky e
    grid $taskbar(seeUrl) $taskbar(conf)    -sticky w
    grid $taskbar(downld)
    grid $taskbar(help) $taskbar(exit) -sticky e
    grid $checks         -sticky ew
    grid $lhtml $htmla $htmle $lima $imaa $imae $lcom $coma $come \
            $ltodo $todoa $todoe $lany $eany $anya $anye -pady 2
    grid $marco          -sticky news -padx 10 -pady 10
    grid $canvas $barray -sticky news
    grid $barrax         -sticky ew

    grid rowconfigure    $win(top) 0 -weight 1
    grid columnconfigure $win(top) 0 -weight 1
    grid rowconfigure    $extFrame 2 -weight 1
    grid columnconfigure $extFrame 0 -weight 1
    grid rowconfigure    $marco    0 -weight 1
    grid columnconfigure $marco    0 -weight 1

    grid columnconfigure $taskbarFrame 0 -weight 1
    grid columnconfigure $taskbarFrame 2 -weight 1

    wm protocol $win(top) WM_DELETE_WINDOW "
            set ChooseLinks::done 0
            destroy $win(top)
    "

    trace variable downOptions(cgi)      w "::ChooseLinks::Traces $linkArray"
    trace variable downOptions(dir)      w "::ChooseLinks::Traces $linkArray"
    trace variable downOptions(external) w "::ChooseLinks::Traces $linkArray"
    trace variable downOptions(images)   w "::ChooseLinks::Traces $linkArray"
    array set traces [array get downOptions]


    # For some reason I need this to initialize the look of TileQt scrollbars
    update

    $win(canvas) config -scrollregion "0 0                                      \
            [winfo reqwidth $win(links)] [winfo reqheight $win(links)]"

    tkwait variable ChooseLinks::done

    trace vdelete downOptions(dir)      w "::ChooseLinks::Traces $linkArray"
    trace vdelete downOptions(external) w "::ChooseLinks::Traces $linkArray"
    trace vdelete downOptions(cgi)      w "::ChooseLinks::Traces $linkArray"
    trace vdelete downOptions(images)   w "::ChooseLinks::Traces $linkArray"

    destroy $win(top)

    return $done
}

}

